//	SB_registerFunction(
//		function name excluding "shapeBuildCommand",
//		usage excluding "@sb name",
//		description of the function
//	);

function SB_registerFunction(%name, %usage, %desc)
{
	if(%name $= "" || %usage $= "" || %desc $= "")
	{
		error("SB_registerFunction - wrong number of arguments.");
		error("usage: (string name, string usage, string description)");
		return;
	}
	if($SB::isFunction[%name])
	{
		error("SB_registerFunction - function already exists.");
		return;
	}

	$SB::FunctionCount = $SB::FunctionCount ? $SB::FunctionCount : 0;

	$SB::isFunction[%name] = true;
	$SB::Function::Name[$SB::FunctionCount] = %name;
	$SB::Function::Usage[$SB::FunctionCount] = %usage;
	$SB::Function::Desc[$SB::FunctionCount] = %desc;

	$SB::FunctionCount++;
}

SB_registerFunction("Circle", "[radius]", "Generates a radius circle around the ghost brick.");
SB_registerFunction("Line", "[x offset] [y offset]", "Generates a line from the ghost brick to the offset coordinates.");

function SB_callFunction(%name, %args)
{
	%func = "shapeBuildCommand" @ %name;
	if(!isFunction(%func))
	{
		return false;
	}

	%args = trim(%args);
	for(%a = 0; %a < getWordCount(%args); %a++)
	{
		%arg = strreplace(getWord(%args, %a), "\"", "");
		%newargs = %newargs @ (%a > 0 ? ", " : "") @ "\"" @ %arg @ "\"";
		// this("that", "that", "that");
	}

	SB_clearCache();
	%eval = %func @ "(" @ %newargs @ ");";
	eval(%eval);
	SB_buildLoop();

	return true;
}

function shapeBuildCommandCircle(%radius)
{
	%x = mClamp(%radius, 1, 1000);
	%y = 0;
	%radiusError = 1-%x;
 
	while( %x >= %y )
	{
		SB_addBrick( %x, %y );
		SB_addBrick( %y, %x );
		SB_addBrick( -%x, %y );
		SB_addBrick( -%y, %x );
		SB_addBrick( -%x, -%y );
		SB_addBrick( -%y, -%x );
		SB_addBrick( %x, -%y );
		SB_addBrick( %y, -%x );
		%y++;
		if( %radiusError < 0 )
		{
			%radiusError += 2 * %y + 1;
		}
		else
		{
			%x--;
			%radiusError += 2 * ( %y - %x + 1 );
		}
	}
}

function shapeBuildCommandLine(%x1, %y1)
{
	if(%x1 < 0)
	{
		%x1 = -%x1;
		%rx = true;
	}
	if(%y1 < 0)
	{
		%y1 = -%y1;
		%ry = true;
	}
	%x0 = %y0 = 0;
	%dx = abs( %x1 - %x0 );
	%sx = %x0 < %x1 ? 1 : -1;
	%dy = abs( %y1 - %y0 );
	%sy = %y0 < %y1 ? 1 : -1; 
	%err = ( %dx > %dy ? %dx : -%dy ) / 2;

	while(true)
	{
		SB_addBrick( ( %rx ? -%x0 : %x0 ), ( %ry ? -%y0 : %y0 ) );
		if ( %x0 == %x1 && %y0 == %y1 ) { break; }
		%e2 = %err;
		if ( %e2 > -%dx ) { %err -= %dy; %x0 += %sx; }
		if ( %e2 < %dy ) { %err += %dx; %y0 += %sy; }
	}
}

function SB_addBrick(%x, %y)
{
	$SB::BrickPos[$SB::BrickCount] = %x SPC %y;
	$SB::BrickCount++;
	echo(%x SPC %y);
}

function SB_clearCache()
{
	deleteVariables("$SB::B*");
}

function SB_buildLoop()
{
	cancel($SB::BuildLoop);
	if($SB::BuiltCount >= $SB::BrickCount || !isObject(ServerConnection))
	{
		return;
	}

	%pos = $SB::BrickPos[$SB::BuiltCount];
	%x = getWord(%pos, 0);
	%y = getWord(%pos, 1);

	commandToServer('superShiftBrick', %x, %y, 0);
	plantBrick(1);
	plantBrick(0);
	commandToServer('superShiftBrick', -%x, -%y, 0);

	$SB::BuiltCount++;

	$SB::BuildLoop = schedule(33, 0, SB_buildLoop);
}

function abs(%val)
{
	return VectorDist(0, %val);
}

if(isPackage(ShapeBuild))
{
	deactivatePackage(ShapeBuild);
}

package ShapeBuild
{
	function NMH_Type::send(%this)
	{
		%value = %this.getValue();
		if(firstWord(%value) $= "@sb")
		{
			%rest = restWords(%value);
			if(%rest $= "")
			{
				NewChatSO.addLine("\c5Shape List:");

				for( %a = 0; %a < $SB::FunctionCount; %a++ )
				{
					NewChatSO.addLine("\c6@sb " @ $SB::Function::Name[%a] SPC $SB::Function::Usage[%a] SPC "-" SPC $SB::Function::Desc[%a]);
				}

				%this.setValue("");
				return Parent::send(%this);
			}

			%name = firstWord(%rest);
			%args = restWords(%rest);
			if(SB_callFunction(%name, %args))
			{
				NewChatSO.addLine("\c5Generating shape ...");
			}
			else
			{
				NewChatSO.addLine("\c5Unknown shape function '" @ %name @ "'. Say \c0@sb \c5for help.");
			}

			%this.setValue("");
			return Parent::send(%this);
		}

		return Parent::send(%this);
	}
};

activatePackage(ShapeBuild);